<?php

namespace Arma\Internal;

/**
 * Interface for all container objects.
 *
 * @package Arma\Internal
 */
interface Common
{
    /**
     * Get number of rows.
     * 
     * @return int
     */
    function nRows();

    /**
     * Get number of columns.
     * 
     * @return int
     */
    function nCols();

    /**
     * Get number of elements.
     * 
     * @return int
     */
    function nElem();

    /**
     * If $j is not specified, access the n-th element. Otherwise, access the element stored at the i-th row
     * and j-th column.
     *
     * @param int $i
     * @param int $j[optional]
     * @return Subview
     */
    function __invoke($i, $j);

    /**
     * If $j is not specified, access the n-th element. Otherwise, access the element stored at the i-th row
     * and j-th column.
     *
     * Without a bounds check. Not recommended for use unless your code has been thoroughly debugged.
     *
     * @param int $i
     * @param int $j[optional]
     * @return Subview
     */
    function at($i, $j);

    /**
     * Addition of two objects.
     *
     * @param mixed $other
     * @return static
     */
    function add($other);

    /**
     * Subtraction of two objects.
     *
     * @param mixed $other
     * @return static
     */
    function sub($other);

    /**
     * Negation of the object.
     *
     * @return static
     */
    function neg();

    /**
     * Matrix multiplication of two objects.
     *
     * @param mixed $other
     * @return static
     */
    function mul($other);

    /**
     * Element-wise multiplication of two objects.
     *
     * @param static $other
     * @return static
     */
    function dotMul($other);

    /**
     * Element-wise division of an object by another object or a scalar.
     *
     * @param mixed $other
     * @return static
     */
    function div($other);

    /**
     * For all elements equal to $old_value, set them to $new_value.
     *
     * For SpMat, replacement is not done when $old_value == 0.
     *
     * @param number|\Arma\Complex $old_value
     * @param number|\Arma\Complex $new_value
     * @return void
     */
    function replace($old_value, $new_value);

    /**
     * Transform each element using a callback function.
     *
     * Transformation is done column-by-column, and for SpMat only non-zero elements will be transformed.
     *
     * @param callable $callback ($old_value) => $new_value
     * @return void
     */
    function transform($callback);

    /**
     * For each element, pass its reference to a callback function.
     *
     * Processing is done column-by-column, and for SpMat only non-zero elements will be passed to callback.
     *
     * @param callable $callback
     * @return void
     */
    function forEach($callback);
}
